#include <math.h>
#include "graphics.h"

#ifndef ROUND
#define ROUND(a) ((int)(a+0.5))
#endif


PhigsState pState = { SOLID, 1, 0, 0, 0, 1, 0, 0, 0, 1 };


/* ============================================================ */
/* Basic graphics primtives */ 
/* ============================================================ */

/* ============================================================
   lineDDA
*/
void lineDDA (int xa, int ya, int xb, int yb)
{
  int dx = xb - xa, dy = yb - ya, steps, k;
  float xIncrement, yIncrement, x = xa, y = ya;

  if (abs (dx) > abs (dy)) 
    steps = abs (dx);
  else 
    steps = abs (dy);
  xIncrement = dx / (float) steps;
  yIncrement = dy / (float) steps;
    
  setPixel (ROUND(x), ROUND(y));
  for (k=0; k<steps; k++) {
    x += xIncrement;
    y += yIncrement;
    setPixel (ROUND(x), ROUND(y));
  }
}



/* ============================================================
   circleMidpoint
*/
void circleMidpoint (int xCenter, int yCenter, int radius)
{
   int x = 0;
   int y = radius;
   int p = 1 - radius;
   void circlePlotPoints (int xCenter, int yCenter, int x, int y);

   circlePlotPoints (xCenter, yCenter, x, y);

   while (x < y) {
      if (p < 0) {
         x++;
         p += 2 * x + 1;
      }
      else {
         x++;
         y--;
         p += 2 * (x - y) + 1;
      }
      circlePlotPoints (xCenter, yCenter, x, y);
   }
}

void circlePlotPoints (int xCenter, int yCenter, int x, int y)
{
   setPixel (xCenter + x, yCenter + y);
   setPixel (xCenter - x, yCenter + y);
   setPixel (xCenter + x, yCenter - y);
   setPixel (xCenter - x, yCenter - y);
   setPixel (xCenter + y, yCenter + x);
   setPixel (xCenter - y, yCenter + x);
   setPixel (xCenter + y, yCenter - x);
   setPixel (xCenter - y, yCenter - x);
}



/* ============================================================
   ellipseMidpoint, p 109-110
*/
void ellipseMidpoint (int xCenter, int yCenter, int Rx, int Ry)
{
  int Rx2 = Rx*Rx, Ry2 = Ry*Ry, twoRx2 = 2*Rx2, twoRy2 = 2*Ry2;
  int p, x = 0, y = Ry, px = 0, py = twoRx2 * y;
  void ellipsePlotPoints (int, int, int, int);

  ellipsePlotPoints (xCenter, yCenter, x, y);

  /* Region 1 */
  p = ROUND (Ry2 - (Rx2*Ry) + (0.25 * Rx2));
  while (px < py) {
    x++;
    px += twoRy2;
    if (p < 0)
      p += Ry2 + px;
    else {
      y--;
      py -= twoRx2;
      p += Ry2 + px - py;
    }
    ellipsePlotPoints (xCenter, yCenter, x, y);
  }

  /* Region 2 */
  p = ROUND (Ry2 * (x+0.5) * (x+0.5) + Rx2 * (y-1) * (y-1) - Rx2 * Ry2);
  while (y > 0) {
    y--;
    py -= twoRx2;
    if (p > 0) 
      p += Rx2 - py;
    else {
      x++;
      px += twoRy2;
      p += Rx2 - py + px;
    }
    ellipsePlotPoints (xCenter, yCenter, x, y);
  }
}

void ellipsePlotPoints (int xCenter, int yCenter, int x, int y)
{
   setPixel (xCenter + x, yCenter + y);
   setPixel (xCenter - x, yCenter + y);
   setPixel (xCenter + x, yCenter - y);
   setPixel (xCenter - x, yCenter - y);
}



/* ============================================================
   scanFill
*/
typedef struct tEdge {
  int yUpper;
  float xIntersect;
  float dxPerScan;
  struct tEdge * next;
} Edge;

/* Inserts edge into list in order of increasing xIntersect field. */
void insertEdge (Edge * list, Edge * edge)
{
  Edge * q = list;
  Edge * p = q->next;
  while (p != NULL) {
    if (edge->xIntersect < p->xIntersect)
      p = NULL;
    else {
      q = p;
      p = p->next;
    }
  }
  edge->next = q->next;
  q->next = edge;
}

/* Given an index, return the y-coordinate of next nonhorizontal line */
int yNext (int k, int cnt, dcPt * pts)
{
  int j;
  if ((k+1) > (cnt-1))
    j = 0;
  else
    j = k + 1;
  while (pts[k].y == pts[j].y)
    if ((j+1) > (cnt-1))
      j = 0;
    else
      j++;
  return (pts[j].y);
}

void makeEdgeRec
  (dcPt lower, dcPt upper, int yComp, Edge * edge, Edge * edges[])
{
  edge->dxPerScan =
    (float) (upper.x - lower.x) / (upper.y - lower.y);
  edge->xIntersect = lower.x;
  if (upper.y < yComp)
    edge->yUpper = upper.y - 1;
  else
    edge->yUpper = upper.y;
  insertEdge (edges[lower.y], edge);
}

void buildEdgeList (int cnt, dcPt * pts, Edge * edges[])
{
  Edge * edge;
  dcPt v1, v2;
  int i;
  int yPrev = pts[cnt - 2].y;
  
  v1.x = pts[cnt-1].x; v1.y = pts[cnt-1].y;
  for (i = 0; i < cnt; i++) {
    v2 = pts[i];
    if (v1.y != v2.y) {  /* a nonhorizontal line */
      edge = (Edge *) malloc (sizeof (Edge));
      if (v1.y < v2.y)
	makeEdgeRec (v1, v2, yNext (i, cnt, pts), edge, edges);
      else
	makeEdgeRec (v2, v1, yPrev, edge, edges);
    }
    yPrev = v1.y;
    v1 = v2;
  }
}

void buildActiveList (int scan, Edge * active, Edge * edges[])
{
  Edge * p, * q;
  p = edges[scan]->next;
  while (p) {
    q = p->next;
    insertEdge (active, p);
    p = q;
  }
}

void fillScan (int scan, Edge * active)
{
  Edge * p1, * p2;
  int i;
  
  p1 = active->next;
  while (p1) {
    p2 = p1->next;
    for (i = p1->xIntersect; i < p2->xIntersect; i++)
      setPixel ((int) i, scan);
    /* Advance to next pair of intersections */
    p1 = p2->next;
  }
}

void deleteAfter (Edge * q)
{
  Edge * p = q->next;
  q->next = p->next;
  free (p);
}

void updateActiveList (int scan, Edge * active)
{
  Edge * p = active->next;
  Edge * q = active;
  while (p) {
    if (scan >= p->yUpper) {
      p = p->next;
      deleteAfter (q);
    }
    else {
      p->xIntersect = p->xIntersect + p->dxPerScan;
      q = p;
      p = p->next;
    }
  }
}

void resortActiveList (Edge * active)
{
  Edge * p = active->next;
  Edge * q;
  active->next = NULL;
  while (p) {
    q = p->next;
    insertEdge (active, p);
    p = q;
  }
}

void scanFill (int cnt, dcPt * pts)
{
  Edge * edges[WINDOW_HEIGHT];
  Edge * active;
  int i, scan;
  active = (Edge *) malloc (sizeof (Edge));
  for (i = 0; i < WINDOW_HEIGHT; i++) {
    edges[i] = (Edge *) malloc (sizeof (Edge));
    edges[i]->next = NULL;
  }
  active->next = NULL;
  buildEdgeList (cnt, pts, edges);
  for (scan = 0; scan < WINDOW_HEIGHT; scan++) {
    buildActiveList (scan, active, edges);
    if (active->next) {
      fillScan (scan, active);
      updateActiveList (scan, active);
      resortActiveList (active);
    }
  }
}



/* ============================================================ */
/* Data structure convenience functions */
/* ============================================================ */

void setWcPt2 (wcPt2 * pt, float x, float y)
{
  pt->x = x; pt->y = y;
}

void setWcPt3 (wcPt3 * pt, float x, float y, float z)
{
  pt->x = x; pt->y = y; pt->z = z;
}

void wcPt2Show (wcPt2 pt)
{
  printf ("(%6.3f, %6.3f)\n", pt.x, pt.y);
}

void wcPt3Show (wcPt3 pt)
{
  printf ("(%6.3f, %6.3f, %6.3f)\n", pt.x, pt.y, pt.z);
}

void matrix3x3Show (pMatrix3 m)
{
  int i, j;

  printf ("\n");
  for (i=0; i<3; i++) {
    for (j=0; j<3; j++)
      printf ("%6.3f\t", m[i][j]);
    printf ("\n");
  }
}

void matrix4x4Show (pMatrix4 m)
{
  int i, j;

  printf ("\n");
  for (i=0; i<4; i++) {
    for (j=0; j<4; j++)
      printf ("%6.3f\t", m[i][j]);
    printf ("\n");
  }
}



/* ============================================================ */
/* PHIGS-like graphics primitives */ 
/* ============================================================ */

void pPoint (float x, float y)
{
  setPixel (ROUND(x), ROUND(y));
}


void pSetLineType (pLineType typ)
{
  pState.lineType = typ;
}


void pPolyline (int nPts, wcPt2 * pts)
{
  int line;
  int dx, dy, steps, k;
  double xIncrement, yIncrement, x, y;
  int on = 0, off = 0;

  if (pState.lineType == SOLID) {
    /* We'll round the points and rely on the device-level line
       drawing routine. */
    for (line=0; line<nPts-1; line++)
      lineDDA ((int) pts[line].x, (int) pts[line].y,
	       (int) pts[line+1].x, (int) pts[line+1].y);
  }
  
  else {
    /* Otherwise we'll have to keep track of which points on the
       line should be "on" and which are "off" */
    for (line=0; line<nPts-1; line++) {
      x = pts[line].x;
      y = pts[line].y;
      dx = pts[line+1].x - x;
      dy = pts[line+1].y - y;
      if (abs (dx) > abs (dy)) 
	steps = abs (dx);
      else
	steps = abs (dy);
      xIncrement = dx / (float) steps;
      yIncrement = dy / (float) steps;
      setPixel ((int) x, (int) y);
      on++;

      switch (pState.lineType) {
      case DASHED: 
	for (k=0; k<steps; k++) {
	  x += xIncrement;
	  y += yIncrement;
	  if (on < 1) {
	    setPixel ((int) x, (int) y);
	    on++;
	  }
	  else {
	    if (off < 2)
	      off++;
	    else {
	      on = 0;
	      off = 0;
	    }
	  }
	}
	break;
      case DOTTED:
	on = 1;
	for (k=0; k<steps; k++) {
	  x += xIncrement;
	  y += yIncrement;
	  if (on < 8) {
	    setPixel ((int) x, (int) y);
	    on++;
	  }
	  else {
	    if (off < 7)
	      off++;
	    else {
	      on = 0;
	      off = 0;
	    }
	  }
	}
	break;
	break;
      }
    }
  }
}


void pPolymarker (int nPts, wcPt2 * pts)
{
  int charWidth = 8;
  int charHeight = 8;
  char str[] = { "*" };
  int i;
  
  for (i = 0; i < nPts; i++) {
    cmov2 (pts[i].x - 0.5 * charWidth, pts[i].y - 0.5 * charHeight);
    charstr (str);
  }
}


void pFillArea (int nPts, wcPt2 * pts) 
{
  dcPt * dcpts;
  int i;
  
  dcpts = (dcPt *) malloc ((nPts+1) * (sizeof (dcPt)));
  
  for (i = 0; i < nPts; i++) {
    dcpts[i].x = (int) pts[i].x;
    dcpts[i].y = (int) pts[i].y;
  }
  dcpts[nPts] = dcpts[0];
  scanFill (nPts+1, dcpts);
  
  free (dcpts);
}


void pText (wcPt2 position, char * outString)
{
  cmov2 (position.x, position.y);
  charstr (outString);
}


void pCircle (wcPt2 center, float radius)
{
  ellipseMidpoint
    (ROUND(center.x), ROUND(center.y), ROUND(radius), ROUND(radius));
}


void pPolyline3 (int nPts, wcPt3 * pts)
{
  float pt[3];
  int i, a;

  bgnline ();
  for (i=0; i<nPts; i++) {
    pt[0] = pts[i].x;
    pt[1] = pts[i].y;
    pt[2] = pts[i].z;
    v3f (pt);
  }
  endline ();
}


void pMatrix3SetIdentity (pMatrix3 m)
{
  int i,j;

  for (i=0; i<3; i++) 
    for (j=0; j<3; j++) 
      m[i][j] = (i == j);
}


void pMatrix3PreMultiply (pMatrix3 m, pMatrix3 t)
{
  int r,c;
  pMatrix3 tmp;

  for (r=0; r<3; r++)
    for (c=0; c<3; c++)
      tmp[r][c] =
	m[r][0]*t[0][c] + m[r][1]*t[1][c] + m[r][2]*t[2][c];

  for (r=0; r<3; r++)
    for (c=0; c<3; c++)
      t[r][c] = tmp[r][c]; 
}


void pScale2 (float sx, float sy, wcPt2 refPt)
{
  pMatrix3 m;

  pMatrix3SetIdentity (m);
  m[0][0] = sx; 
  m[0][2] = (1 - sx) * refPt.x;
  m[1][1] = sy; 
  m[1][2] = (1 - sy) * refPt.x;
  pMatrix3PreMultiply (m, pState.ctm);
}


float pToRadians (float a)
{ 
  return (a * 3.14159 / 180.0);
}


void pRotate2 (float a, wcPt2 refPt)
{
  pMatrix3 m;

  pMatrix3SetIdentity (m);
  a = pToRadians (a);
  m[0][0] = cosf(a);
  m[0][1] = -sinf(a);
  m[0][2] = refPt.x * (1 - cosf (a)) + refPt.y * sinf (a);
  m[1][0] = sinf(a);
  m[1][1] = cosf(a);
  m[1][2] = refPt.y * (1 - cosf (a)) - refPt.y * sinf (a);
  pMatrix3PreMultiply (m, pState.ctm);
}


void pTranslate2 (int tx, int ty)
{
  pMatrix3 m;

  pMatrix3SetIdentity (m);
  m[0][2] = tx;
  m[1][2] = ty;
  pMatrix3PreMultiply (m, pState.ctm);
}


void pTransformPoints2 (int npts, wcPt2 * pts)
{
  int k;
  float tmp;
  
  for (k = 0; k < npts; k++) {
    tmp = pState.ctm[0][0] * pts[k].x + pState.ctm[0][1] * 
      pts[k].y + pState.ctm[0][2];
    pts[k].y = pState.ctm[1][0] * pts[k].x + pState.ctm[1][1] * 
      pts[k].y + pState.ctm[1][2];
    pts[k].x = tmp;
  }
}


