/*  mandelbrot, Chapter 10, p. 384  */

#include <stdio.h>
#include <stdlib.h>

/* EXAMPLE STARTS HERE */
#include "graphics.h"

typedef struct { float x, y; } Complex;

Complex complexSquare (Complex c)
{
  Complex cSq;

  cSq.x = c.x * c.x - c.y * c.y;
  cSq.y = 2 * c.x * c.y;
  return (cSq);
}

int iterate (Complex zInit, int maxIter)
{
  Complex z = zInit;
  int cnt = 0;

  /*  Quit when z * z > 4  */
  while ((z.x * z.x + z.y * z.y <= 4.0) && (cnt < maxIter)) {
    z = complexSquare (z);
    z.x += zInit.x;
    z.y += zInit.y;
    cnt++;
  }
  return (cnt);
}

void mandelbrot (int nx, int ny, int maxIter, float realMin,
		 float realMax, float imagMin, float imagMax)
{
  float realInc = (realMax - realMin) / nx;
  float imagInc = (imagMax - imagMin) / ny;
  Complex z;
  int x, y;
  int cnt;

  for (x=0, z.x=realMin; x<nx; x++, z.x+=realInc) 
    for (y=0, z.y=imagMin; y<ny; y++, z.y+=imagInc) {
      cnt = iterate (z, maxIter);
      if (cnt == maxIter)
	setColor (BLACK);
      else
	setColor (cnt);
      pPoint (x, y);
    }
}
/* EXAMPLE ENDS HERE */

#define WINDOW_WIDTH 300
#define WINDOW_HEIGHT 300

void main (int argc, char ** argv)
{
  long windowID;
  float realMin, realMax, imagMin, imagMax;
  char winName[128];
  int nIter = 0;

  if (argc < 6) {
    fprintf (stderr,
	     "Usage: \t%s realMin realMax, imagMin, imagMax nIterations\n",
	     argv[0]);
    fprintf (stderr, "Try: \t%s -2.25 0.75 -1.25 1.25 128\n", argv[0]);
    exit (-1);
  }

  sprintf (winName, "%s", argv[0]);

  argv++;
  realMin = atof (*argv++);
  realMax = atof (*argv++);
  imagMin = atof (*argv++);
  imagMax = atof (*argv++);
  nIter = atoi (*argv++);
  if (nIter <= 0) {
    fprintf (stderr, "HINT: Number of iterations must be positive.\n");
    fprintf (stderr, "HINT: Setting to 10.  More would show greater detail\n");
    nIter = 10;
  }

  windowID = openGraphics (winName, WINDOW_WIDTH, WINDOW_HEIGHT);
  /* Clear the window */
  setBackground (BLACK);

  mandelbrot
    (WINDOW_WIDTH, WINDOW_HEIGHT, nIter, realMin, realMax, imagMin, imagMax);

  sleep (10);
  closeGraphics (windowID);
}



