/* octree, Chapter 13, pp. 486 - 487 */

#include <stdio.h>

/* EXAMPLE STARTS HERE */
typedef enum { SOLID, MIXED } Status;

#define EMPTY -1

typedef struct tOctree {
  int id;
  Status status;
  union {
    int color;
    struct tOctree * children[8];    
  } data;
} Octree;

typedef struct tQuadtree {
  int id;
  Status status;
  union {
    int color;
    struct tQuadtree * children[4];
  } data;
} Quadtree;

int nQuadtree = 0;

void octreeToQuadtree (Octree * oTree, Quadtree * qTree)
{
  Octree * front, * back;
  Quadtree * newQuadtree;
  int i, j;
  
  if (oTree->status == SOLID) {
    qTree->status = SOLID;
    qTree->data.color = oTree->data.color;
    return;
  }
  qTree->status = MIXED;
  /* Fill in each quad of the quadtree */
  for (i=0; i<4; i++) {
    front = oTree->data.children[i];
    back = oTree->data.children[i+4];
    newQuadtree = (Quadtree *) malloc (sizeof (Quadtree));
    newQuadtree->id = nQuadtree++;
    newQuadtree->status = SOLID;
    qTree->data.children[i] = newQuadtree;

    if (front->status == SOLID) 
      if (front->data.color != EMPTY) 
	qTree->data.children[i]->data.color = front->data.color;
      else 
	if (back->status == SOLID) 
	  if (back->data.color != EMPTY)
	    qTree->data.children[i]->data.color = back->data.color;
	  else
	    qTree->data.children[i]->data.color = EMPTY;
	else { /* back node is mixed */
	  newQuadtree->status = MIXED;
	  octreeToQuadtree (back, newQuadtree);
	}
    else { /* front node is mixed */
      newQuadtree->status = MIXED;
      octreeToQuadtree (back, newQuadtree);
      octreeToQuadtree (front, newQuadtree);
    }
  }
}
/* EXAMPLE ENDS HERE */


void dumpOctree (Octree tree)
{
  int i;
  static int level = -1;

  level++;
  
  for (i=0; i<level; i++)
    printf ("\t");
  switch (tree.status) {
  case SOLID:
    if (tree.data.color == EMPTY)
      printf ("%2d empty\n", tree.id);
    else
      printf ("%2d %d\n", tree.id, tree.data.color);
    break;
  case MIXED:
    printf ("%2d mixed\n", tree.id);
    for (i=0; i<8; i++)
      dumpOctree (*tree.data.children[i]);
    break;
  }
  level--;
}

void dumpQuadtree (Quadtree tree)
{
  int i;
  static int level = -1;

  level++;
  
  for (i=0; i<level; i++)
    printf ("\t");
  switch (tree.status) {
  case SOLID:
    if (tree.data.color == EMPTY)
      printf ("%2d empty\n", tree.id);
    else
      printf ("%2d %d\n", tree.id, tree.data.color);
    break;
  case MIXED:
    printf ("%2d mixed\n", tree.id);
    for (i=0; i<4; i++)
      dumpQuadtree (*tree.data.children[i]);
    break;
  }
  level--;
}

#define TOP 0
#define N_NODES 25
#define N_MIXED 3
#define RED 4

void main ()
{
  int i, j, indx;
  Quadtree * quadtree;
  Octree octree[N_NODES];
  int mixedNodes[N_MIXED] = { 0, 3, 8 };
  int s[N_NODES] = {
    MIXED, 
    SOLID, EMPTY, MIXED, EMPTY, EMPTY, SOLID, SOLID, MIXED,
    SOLID, EMPTY, EMPTY, EMPTY, EMPTY, EMPTY, SOLID, SOLID, 
    EMPTY, EMPTY, EMPTY, SOLID, EMPTY, SOLID, SOLID, EMPTY };
  
  /* Set the status flags and color fields */
  for (i=0; i<N_NODES; i++) {
    octree[i].id = i;
    octree[i].status = s[i];
    if (octree[i].status == SOLID)
      octree[i].data.color = RED;
    else
      if (octree[i].status == EMPTY) {
	octree[i].status = SOLID;
	octree[i].data.color = EMPTY;
      }
  }

  /* Use the nodes to make up an octree */
  indx = 1;
  for (i=0; i<N_MIXED; i++)
    for (j=0; j<8; j++)
      octree[mixedNodes[i]].data.children[j] = &octree[indx++];

  printf ("dumping initial octree\n");
  dumpOctree (octree[TOP]);

  printf ("\n\nconverting octree to quadtree\n");
  quadtree = (Quadtree *) malloc (sizeof (Quadtree));
  quadtree->id = nQuadtree++;
  octreeToQuadtree (&octree[TOP], quadtree);

  printf ("\ndumping quadtree\n");
  dumpQuadtree (*quadtree);
}
