/* selfSquare, Chapter 10, pp. 380-381  */

#include <stdio.h>
#include <stdlib.h>

/* EXAMPLE STARTS HERE */
#include <math.h>
#include <values.h>
#include "graphics.h"

typedef struct {
  float x, y;
} Complex;

void calculatePoint (Complex lambda, Complex * z)
{
  float lambdaMagSq, discrMag;
  Complex discr;
  static Complex fourOverLambda = { 0, 0 };
  static firstPoint = TRUE;
    
  if (firstPoint) {
    /* Compute 4 divided by lambda */
    lambdaMagSq = lambda.x * lambda.x + lambda.y * lambda.y;
    fourOverLambda.x =  4 * lambda.x / lambdaMagSq;
    fourOverLambda.y = -4 * lambda.y / lambdaMagSq;
    firstPoint = FALSE;
  }
  discr.x = 1.0 - (z->x * fourOverLambda.x - z->y * fourOverLambda.y);
  discr.y = z->x * fourOverLambda.y + z->y * fourOverLambda.x;
  discrMag = sqrt (discr.x * discr.x + discr.y * discr.y);

  /* Update z, checking to avoid the sqrt of a negative number */
  if (discrMag + discr.x < 0)
    z->x = 0;
  else
    z->x = sqrt ((discrMag + discr.x) / 2.0);
  if (discrMag - discr.x < 0)
    z->y = 0;
  else 
    z->y = 0.5 * sqrt ((discrMag - discr.x) / 2.0);

  /* For half the points, use negative root, placing point in quadrant 3 */
  if (random() < MAXINT/2) {
    z->x = -z->x;
    z->y = -z->y;
  }

  /* When imaginary part of discriminant is negative, point
     should lie in quadrant 2 or 4, so reverse sign of x */
  if (discr.y < 0) z->x = -z->x;

  /* Finish up calculation for the real part of z */
  z->x = 0.5 * (1 - z->x);
}

void selfSquare (Complex lambda, Complex z, int count)
{
  int k;

  /* Skip the first few points */
  for (k=0; k<10; k++)
    calculatePoint (lambda, &z);
  for (k=0; k<count; k++) {
    calculatePoint (lambda, &z);
    /* Scale point to fit window and draw */
    pPoint (z.x*WINDOW_WIDTH, 0.5*WINDOW_HEIGHT+z.y*WINDOW_HEIGHT);
  }
}
/* EXAMPLE ENDS HERE */

void main (int argc, char ** argv)
{
  long windowID;
  Complex lambda, z;
  char winName[128];
  int nPts;

  if (argc < 6) {
    fprintf
      (stderr, "Usage: \t%s lambdaReal lambdaImag zReal zImag nPoints\n",
       argv[0]);
    fprintf
      (stderr, "Try: \t%s 3 0 0 0 2500 or \n\t%s 2 1 0 0 5000\n",
       argv[0], argv[0]);
    exit (-1);
  }
  sprintf (winName, "%s: lambda is (%s, %s)", argv[0], argv[1], argv[2]);

  argv++;
  lambda.x = atof (*argv++);
  lambda.y = atof (*argv++);
  z.x = atof (*argv++);
  z.y = atof (*argv++);
  nPts = atoi (*argv++);
  if (nPts < 1000) {
    fprintf
      (stderr, "HINT: %d points will look too sparse. Resetting to 1000\n");
    nPts = 1000;
  }

  windowID = openGraphics (winName, WINDOW_WIDTH, WINDOW_HEIGHT);
  setBackground (WHITE);
  setColor (BLACK);
  selfSquare (lambda, z, nPts);

  sleep (10);
  closeGraphics (windowID);
}


